#!/bin/env python

import argparse
import re
import sys
import gtk
import codecs

from gwsmhg_pkg import patchlib

PARSER = argparse.ArgumentParser(description='Check file(s) for UTF-8 compliance.')

PARSER.add_argument(
    'arg_file_list',
    help='The name of the files to be processed.',
    metavar='file',
    nargs='*',
)

ARGS = PARSER.parse_args()

def is_utf8_compliant(text):
    try:
        _ = text.decode('utf-8')
    except UnicodeError:
        return False
    return True

ISO_8859_CODECS = ['iso-8859-{0}'.format(x) for x in range(1, 17)]
ISO_2022_CODECS = ['iso-2022-jp', 'iso-2022-kr'] + \
    ['iso-2022-jp-{0}'.format(x) for x in range(1, 3) + ['2004', 'ext']]

def make_utf8_compliant(text):
    if is_utf8_compliant(text):
        return text
    for codec in ISO_8859_CODECS + ISO_2022_CODECS:
        try:
            text = unicode(text, codec).encode('utf-8')
            return text
        except UnicodeError:
            continue
    raise UnicodeError

problem_files = []
for path in ARGS.arg_file_list:
    text = open(path, 'rb').read()
    patch = patchlib.Patch.parse_text(text)
    text = str(patch.header)
    num_bad_lines = 0
    text_lines = text.splitlines(True)
    mtext_lines = text_lines[:]
    for line_num in range(len(text_lines)):
        if not is_utf8_compliant(text_lines[line_num]):
            print 'BAD[%s]:' % line_num, text_lines[line_num]
            mtext_lines[line_num] = make_utf8_compliant(text_lines[line_num])
            num_bad_lines += 1
    if num_bad_lines:
        problem_files.append(path)
        print 'File: {0}; has {1} badlines'.format(path, num_bad_lines)
        import difflib
        for line in difflib.unified_diff(text_lines, mtext_lines):
            print line,

if problem_files:
    print len(problem_files), 'Problem Files', problem_files
